import pytest
from votesmart.methods import utils as uts
from votesmart.methods.containers import VotesmartApiObject

# import _parse_api_response, VotesmartApiError

def test_sanity():
    assert 1 + 1 == 2

def test_parse_api_response():
    response = {"noerror": "here"}
    value = uts.parse_api_response(response)
    assert response == value

def test_parse_api_response_exception():
    with pytest.raises(uts.VotesmartApiError):
        response = {"error": {"errorMessage": "a message"}}
        value = uts.parse_api_response(response)



from .address import Address
from .candidatebio import CandidateBio
from .candidates import Candidates
from .committee import Committee
from .district import District
from .elections import Election
from .leadership import Leadership
from .local import Local
from .measure import Measure
from .npat import Npat
from .office import Office
from .officials import Officials
from .ratings import Rating
from .states import State
from .votes import Votes



"""
Local methods that correspond to this API documentation page

http://api.votesmart.org/docs/Local.html
"""

from .base import APIMethodBase

class Local(APIMethodBase):
    def __init__(self, api_instance):
        raise NotImplementedError()

#     class Local(object):
#         @staticmethod
#         def getCounties(stateId):
#             params = {'stateId': stateId}
#             result = votesmart._apicall('Local.getCounties', params)
#             return _result_to_obj(Locality, result['counties']['county'])
#
#         @staticmethod
#         def getCities(stateId):
#             params = {'stateId': stateId}
#             result = votesmart._apicall('Local.getCities', params)
#             return _result_to_obj(Locality, result['cities']['city'])
#
#         @staticmethod
#         def getOfficials(localId):
#             params = {'localId': localId}
#             result = votesmart._apicall('Local.getOfficials', params)
#             return _result_to_obj(Official, result['candidateList']['candidate'])



from votesmart.methods.containers import *

def test_sanity():
    assert 1 + 1 == 2

def test_VotesmartApiObject():
    data = {"hello": "friend"}
    result = VotesmartApiObject(data)
    assert result.hello == "friend"



"""
Committee methods that correspond to this API documentation page

http://api.votesmart.org/docs/Committee.html
"""

from .base import APIMethodBase

class Committee(APIMethodBase):
    def __init__(self, api_instance):
        raise NotImplementedError()
#     class Committee(object):
#         @staticmethod
#         def getTypes():
#             result = votesmart._apicall('Committee.getTypes', {})
#             return _result_to_obj(CommitteeType, result['committeeTypes']['type'])
#
#         @staticmethod
#         def getCommitteesByTypeState(typeId=None, stateId=None):
#             params = {'typeId':typeId, 'stateId':stateId}
#             result = votesmart._apicall('Committee.getCommitteesByTypeState', params)
#             return _result_to_obj(Committee, result['committees']['committee'])
#
#         @staticmethod
#         def getCommittee(committeeId):
#             params = {'committeeId' : committeeId}
#             result = votesmart._apicall('Committee.getCommittee', params)
#             return CommitteeDetail(result['committee'])
#
#         @staticmethod
#         def getCommitteeMembers(committeeId):
#             params = {'committeeId' : committeeId}
#             result = votesmart._apicall('Committee.getCommitteeMembers', params)
#             return _result_to_obj(CommitteeMember, result['committeeMembers']['member'])



from ..exceptions import VotesmartApiError

def parse_api_response(response):
    if response.get('error'):
        raise VotesmartApiError(response['error']['errorMessage'])
    else:
        return response

def result_to_obj(cls, result):
    "Convert a dict / list response into a list of parsed elements"
    if isinstance(result, dict):
        return [cls(result)]
    else:
        # the if o predicate is important, sometimes they return empty strings
        return [cls(o) for o in result if o]



"""
Office methods that correspond to this API documentation page

http://api.votesmart.org/docs/Office.html
"""

from .base import APIMethodBase
from .containers import VotesmartApiObject, Candidate
from .utils import result_to_obj

class OfficeData(VotesmartApiObject):
    def __str__(self):
        return self.name

class OfficeType(VotesmartApiObject):
    def __str__(self):
        return ': '.join((self.officeTypeId, self.name))

class OfficeBranch(VotesmartApiObject):
    def __str__(self):
        return ': '.join((self.officeBranchId, self.name))

class OfficeLevel(VotesmartApiObject):
    def __str__(self):
        return ': '.join((self.officeLevelId, self.name))


class Office(APIMethodBase):

    def getTypes(self):
        result = self.api.api_call('Office.getTypes', {})
        return self.result_to_obj(OfficeType, result['officeTypes']['type'])

    def getBranches(self):
        result = self.api.api_call('Office.getBranches', {})
        return self.result_to_obj(OfficeBranch, result['branches']['branch'])

    def getLevels(self):
        result = self.api.api_call('Office.getLevels', {})
        return self.result_to_obj(OfficeLevel, result['levels']['level'])

    def getOfficesByType(self, typeId):
        params = {'officeTypeId':typeId}
        result = self.api.api_call('Office.getOfficesByType', params)
        return self.result_to_obj(OfficeData, result['offices']['office'])

    def getOfficesByLevel(self, levelId):
        params = {'levelId':levelId}
        result = self.api.api_call('Office.getOfficesByLevel', params)
        return self.result_to_obj(OfficeData, result['offices']['office'])

    def getOfficesByTypeLevel(self, officeTypeId, levelId):
        params = {'officeTypeId':officeTypeId, 'officeLevelId':levelId}
        result = self.api.api_call('Office.getOfficesByTypeLevel', params)
        return self.result_to_obj(OfficeData, result['offices']['office'])

    def getOfficesByBranchLevel(self, branchId, levelId):
        params = {'branchId':branchId, 'levelId':levelId}
        result = self.api.api_call('Office.getOfficesByBranchLevel', params)
        return self.result_to_obj(OfficeData, result['offices']['office'])



"""
State methods that correspond to this API documentation page

http://api.votesmart.org/docs/State.html
"""


from .base import APIMethodBase
from .containers import VotesmartApiObject

class StateData(VotesmartApiObject):
    def __str__(self):
        return ' '.join((self.stateId, self.name))

class StateDetail(VotesmartApiObject):
    def __str__(self):
        return ' '.join((self.stateId, self.name))

class State(APIMethodBase):

    def getStateIDs(self):
        result = self.api.api_call('State.getStateIDs', {})
        return self.result_to_obj(StateData, result['stateList']['list']['state'])

    def getState(self, stateId):
        params = {'stateId' : stateId}
        result = self.api.api_call('State.getState', params)
        return StateDetail(result['state']['details'])



import os

import requests
import six

from .methods.utils import parse_api_response

from .exceptions import VotesmartApiError
from . import methods

class VoteSmartAPI:
    def __init__(self, api_key=None):
        if api_key is None:
            raise ValueError('A Votesmart api_key is required')
        self.api_key = api_key

    def api_call(self, function, params):
        request_str = "http://api.votesmart.org/{function}".format(function=function)
        payload = self._set_payload(params)
        response = requests.get(request_str, params=payload)
        try:
            parsed_data = parse_api_response(response.json())
            return parsed_data
        except ValueError:
            raise VotesmartApiError('Problem with API Response')


    def _set_payload(self, params):
        params = params.copy()
        params.update({'key': self.api_key, "o":"JSON"})
        return params

    @property
    def Address(self):
        return methods.Address(self)

    @property
    def CandidateBio(self):
        return methods.CandidateBio(self)

    @property
    def Candidates(self):
        return methods.Candidates(self)

    @property
    def Committee(self):
        return methods.Committee(self)

    @property
    def District(self):
        return methods.District(self)

    @property
    def Election(self):
        return methods.Election(self)

    @property
    def Leadership(self):
        return methods.Leadership(self)

    @property
    def Local(self):
        return methods.Local(self)

    @property
    def Measure(self):
        return methods.Measure(self)

    @property
    def Npat(self):
        return methods.Npat(self)

    @property
    def Office(self):
        return methods.Office(self)

    @property
    def Officials(self):
        return methods.Officials(self)

    @property
    def State(self):
        return methods.State(self)

    @property
    def Rating(self):
        return methods.Rating(self)

    @property
    def Votes(self):
        return methods.Votes(self)



"""
Candidates methods that correspond to this API documentation page

http://api.votesmart.org/docs/Candidates.html
"""


from .base import APIMethodBase
from .containers import Candidate


class Candidates(APIMethodBase):

    def getByOfficeState(self, officeId, stateId=None, electionYear=None, stageId=None):
        params = {'officeId': officeId, 'stateId':stateId, 'electionYear': electionYear, "stageId": stageId}
        result = self.api.api_call('Candidates.getByOfficeState', params)
        return self.result_to_obj(Candidate, result['candidateList']['candidate'])

    def getByOfficeTypeState(self, officeTypeId, stateId=None, electionYear=None):
        params = {'officeTypeId': officeTypeId, 'stateId':stateId, 'electionYear': electionYear}
        result = self.api.api_call('Candidates.getByOfficeTypeState', params)
        return self.result_to_obj(Candidate, result['candidateList']['candidate'])

    def getByLastname(self, lastName, electionYear=None):
        params = {'lastName': lastName, 'electionYear':electionYear}
        result = self.api.api_call('Candidates.getByLastname', params)
        return self.result_to_obj(Candidate, result['candidateList']['candidate'])

    def getByLevenstein(self, lastName, electionYear=None):
        params = {'lastName': lastName, 'electionYear':electionYear}
        result = self.api.api_call('Candidates.getByLevenstein', params)
        return self.result_to_obj(Candidate, result['candidateList']['candidate'])

    def getByElection(self, electionId, stageId=None):
        params = {'electionId': electionId, 'stageId': stageId}
        result = self.api.api_call('Candidates.getByElection', params)
        return self.result_to_obj(Candidate, result['candidateList']['candidate'])

    def getByDistrict(self, districtId, electionYear=None):
        params = {'districtId': districtId, 'electionYear':electionYear}
        result = self.api.api_call('Candidates.getByDistrict', params)
        return self.result_to_obj(Candidate, result['candidateList']['candidate'])

    def getByZip(self, zip5, electionYear=None, zip4=None):
        # errors on zip 20001
        params = {'zip4': zip4, 'zip5': zip5, 'electionYear':electionYear}
        result = self.api.api_call('Candidates.getByZip', params)
        return self.result_to_obj(Candidate, result['candidateList']['candidate'])



"""
Election methods that correspond to this API documentation page

http://api.votesmart.org/docs/Election.html
"""

from .base import APIMethodBase
from .containers import VotesmartApiObject, Candidate
from .utils import result_to_obj

class ElectionData(VotesmartApiObject):
    def __init__(self, d):
        stages = d.pop('stage', None)
        self.__dict__ = d
        if stages:
            self.stages = result_to_obj(ElectionStage, stages)

    def __str__(self):
        return self.name

class ElectionStage(VotesmartApiObject):
    def __str__(self):
        return '%s (%s)' % (self.name, self.electionDate)


class Election(APIMethodBase):

    def getElection(self, electionId):
        params = {'electionId':electionId}
        result = self.api.api_call('Election.getElection', params)
        return ElectionData(result['elections']['election'])

    def getElectionByYearState(self, year, stateId=None):
        params = {'year':year, 'stateId':stateId}
        result = self.api.api_call('Election.getElectionByYearState', params)
        return self.result_to_obj(ElectionData, result['elections']['election'])

    def getElectionByZip(self, zip5, zip4=None, year=None):

        params = {'zip5': zip5, 'zip4': zip4, 'year': year}
        result = self.api.api_call('Election.getElectionByZip', params)
        return self.result_to_obj(ElectionData, result['elections']['election'])

    def getStageCandidates(self, electionId, stageId, party=None,
                           districtId=None, stateId=None):

        params = {'electionId':electionId, 'stageId':stageId,
                  'party':party, 'districtId':districtId, 'stateId':stateId}
        result = self.api.api_call('Election.getStageCandidates', params)
        return self.result_to_obj(Candidate, result['stageCandidates']['candidate'])



"""
Address methods that correspond to this API documentation page

http://api.votesmart.org/docs/Address.html
"""


from .base import APIMethodBase


class Address(APIMethodBase):
    def __init__(self, api_instance):
        raise NotImplementedError()

#         @staticmethod
#         def getCampaign(candidateId):
#             params = {'candidateId': candidateId}
#             result = votesmart._apicall('Address.getCampaign', params)
#             return _result_to_obj(AddressData, result['address']['office'])
#
#         @staticmethod
#         def getCampaignWebAddress(candidateId):
#             params = {'candidateId': candidateId}
#             result = votesmart._apicall('Address.getCampaignWebAddress', params)
#             return _result_to_obj(WebAddress, result['webaddress']['address'])
#
#         @staticmethod
#         def getCampaignByElection(electionId):
#             params = {'electionId': electionId}
#             result = votesmart._apicall('Address.getCampaignByElection', params)
#             return _result_to_obj(AddressData, result['address']['office'])
#
#         @staticmethod
#         def getOffice(candidateId):
#             params = {'candidateId': candidateId}
#             result = votesmart._apicall('Address.getOffice', params)
#             return _result_to_obj(AddressData, result['address']['office'])
#
#         @staticmethod
#         def getOfficeWebAddress(candidateId):
#             params = {'candidateId': candidateId}
#             result = votesmart._apicall('Address.getOfficeWebAddress', params)
#             return _result_to_obj(WebAddress, result['webaddress']['address'])
#
#         #@staticmethod
#         #def getOfficeByOfficeState(officeId, stateId=None):
#         #    params = {'officeId': officeId, 'stateId': stateId}
#         #    result = votesmart._apicall('Address.getOfficeByOfficeState', params)
#         #    return _result_to_obj(Address, result['address']['office'])



import pytest
from votesmart.methods.candidatebio import *

def test_CandidateBio():
    method = CandidateBio(api_instance='test')


def test_Bio():
    d = {"candidate": {"items": 'ok', "something": 'value'}, 'story': "blah"}
    obj = Bio(d)


def test_AddlBio():
    AddlBio({"candidate": 'hello'})



"""
Votes methods that correspond to this API documentation page

http://api.votesmart.org/docs/Votes.html
"""


from .base import APIMethodBase

class Votes(APIMethodBase):
    def __init__(self, api_instance):
        raise NotImplementedError()

#     class Votes(object):
#         @staticmethod
#         def getCategories(year, stateId=None):
#             params = {'year':year, 'stateId':stateId}
#             result = votesmart._apicall('Votes.getCategories', params)
#             return _result_to_obj(Category, result['categories']['category'])
#
#         @staticmethod
#         def getBill(billId):
#             params = {'billId':billId}
#             result = votesmart._apicall('Votes.getBill', params)
#             return BillDetail(result['bill'])
#
#         @staticmethod
#         def getBillAction(actionId):
#             params = {'actionId':actionId}
#             result = votesmart._apicall('Votes.getBillAction', params)
#             return BillActionDetail(result['action'])
#
#         @staticmethod
#         def getBillActionVotes(actionId):
#             params = {'actionId':actionId}
#             result = votesmart._apicall('Votes.getBillActionVotes', params)
#             return _result_to_obj(Vote, result['votes']['vote'])
#
#         @staticmethod
#         def getBillActionVoteByOfficial(actionId, candidateId):
#             params = {'actionId':actionId, 'candidateId':candidateId}
#             result = votesmart._apicall('Votes.getBillActionVoteByOfficial', params)
#             return Vote(result['votes']['vote'])
#
#         @staticmethod
#         def getByBillNumber(billNumber):
#             params = {'billNumber': billNumber}
#             result = votesmart._apicall('Votes.getByBillNumber', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getBillsByCategoryYearState(categoryId, year, stateId=None):
#             params = {'categoryId':categoryId, 'year':year, 'stateId':stateId}
#             result = votesmart._apicall('Votes.getBillsByCategoryYearState', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getBillsByYearState(year, stateId=None):
#             params = {'year':year, 'stateId':stateId}
#             result = votesmart._apicall('Votes.getBillsByYearState', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getBillsByOfficialYearOffice(candidateId, year, officeId=None):
#             params = {'candidateId':candidateId, 'year':year, 'officeId':officeId}
#             result = votesmart._apicall('Votes.getBillsByOfficialYearOffice', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getBillsByOfficial(candidateId, year, officeId=None, categoryId=None):
#             params = {'candidateId':candidateId, 'year':year, 'officeId':officeId, 'categoryId':categoryId}
#             result = votesmart._apicall('Votes.getBillsByOfficial', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getBillsByOfficialCategoryOffice(candidateId, categoryId, officeId=None):
#             params = {'candidateId':candidateId, 'categoryId':categoryId, 'officeId':officeId}
#             result = votesmart._apicall('Votes.getBillsByOfficialCategoryOffice', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getBillsBySponsorYear(candidateId, year):
#             params = {'candidateId':candidateId, 'year':year}
#             result = votesmart._apicall('Votes.getBillsBySponsorYear', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getBillsBySponsorCategory(candidateId, categoryId):
#             params = {'candidateId':candidateId, 'categoryId':categoryId}
#             result = votesmart._apicall('Votes.getBillsBySponsorCategory', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getBillsByStateRecent(stateId=None, amount=None):
#             params = {'stateId':stateId, 'amount':amount}
#             result = votesmart._apicall('Votes.getBillsByStateRecent', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getVetoes(candidateId):
#             params = {'candidateId': candidateId}
#             result = votesmart._apicall('Votes.getVetoes', params)
#             return _result_to_obj(Veto, result['vetoes']['veto'])



"""Containers to hold individual data items in a response
"""

class VotesmartApiObject(dict):
    "Dictionary like object"

    def __init__(self, d):
        self.__dict__ = d

    def __setitem__(self, key, item):
        self.__dict__[key] = item

    def __getitem__(self, key):
        return self.__dict__[key]

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, self.__dict__)

    def __len__(self):
        return len(self.__dict__)

    def __delitem__(self, key):
        del self.__dict__[key]

    def clear(self):
        return self.__dict__.clear()

    def copy(self):
        return self.__dict__.copy()

    def has_key(self, k):
        return k in self.__dict__

    def get(self, k, default=False):
        if self.has_key(k):
            return self.__getitem__(k)
        else:
            return default

    def update(self, *args, **kwargs):
        return self.__dict__.update(*args, **kwargs)

    def keys(self):
        return self.__dict__.keys()

    def values(self):
        return self.__dict__.values()

    def items(self):
        return self.__dict__.items()

    def pop(self, *args):
        return self.__dict__.pop(*args)

    def __cmp__(self, dict_):
        return self.__cmp__(self.__dict__, dict_)

    def __contains__(self, item):
        return item in self.__dict__

    def __iter__(self):
        return iter(self.__dict__)

    def __unicode__(self):
        return unicode(repr(self.__dict__))


class Candidate(VotesmartApiObject):
    def __str__(self):
        return ' '.join((self.firstName, self.lastName))

    def get(self, k, default=False):
        if self.has_key(k):
            return self.__getitem__(k)
        else:
            return default



"""
Officials methods that correspond to this API documentation page

http://api.votesmart.org/docs/Officials.html
"""

from .base import APIMethodBase

class Officials(APIMethodBase):
    def __init__(self, api_instance):
        raise NotImplementedError()

#     class Officials(object):
#         @staticmethod
#         def getStatewide(stateId=None):
#             params = {'stateId': stateId}
#             result = votesmart._apicall('Officials.getStatewide', params)
#             return _result_to_obj(Official, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByOfficeState(officeId, stateId=None):
#             params = {'officeId':officeId, 'stateId': stateId}
#             result = votesmart._apicall('Officials.getByOfficeState', params)
#             return _result_to_obj(Official, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByLastname(lastName):
#             params = {'lastName':lastName}
#             result = votesmart._apicall('Officials.getByLastname', params)
#             return _result_to_obj(Official, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByLevenstein(lastName):
#             params = {'lastName':lastName}
#             result = votesmart._apicall('Officials.getByLevenstein', params)
#             return _result_to_obj(Official, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByElection(electionId):
#             params = {'electionId':electionId}
#             result = votesmart._apicall('Officials.getByElection', params)
#             return _result_to_obj(Official, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByDistrict(districtId):
#             params = {'districtId':districtId}
#             result = votesmart._apicall('Officials.getByDistrict', params)
#             return _result_to_obj(Official, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByZip(zip5, zip4=None):
#             params = {'zip4': zip4, 'zip5': zip5}
#             result = votesmart._apicall('Officials.getByZip', params)
#             return _result_to_obj(Official, result['candidateList']['candidate'])



"""
Npat methods that correspond to this API documentation page

Political Courage Test

Returns a candidates most recently filled out Political Courage Test.

http://api.votesmart.org/docs/Npat.html
"""

from .base import APIMethodBase

class Npat(APIMethodBase):
    def __init__(self, api_instance):
        raise NotImplementedError()
#     class Npat(object):
#         @staticmethod
#         def getNpat(candidateId):
#             params = {'candidateId':candidateId}
#             result = votesmart._apicall('Npat.getNpat', params)
#             return result['npat']



"""
District methods that correspond to this API documentation page

http://api.votesmart.org/docs/District.html
"""

from .base import APIMethodBase
from .containers import VotesmartApiObject

class DistrictData(VotesmartApiObject):
    def __str__(self):
        return self.name

class District(APIMethodBase):

    def getByOfficeState(self, officeId, stateId, districtName=None):
        params = {'officeId':officeId, 'stateId': stateId, 'districtName': districtName}
        result = self.api.api_call('District.getByOfficeState', params)
        return self.result_to_obj(DistrictData, result['districtList']['district'])

    def getByZip(self, zip5, zip4=None):
        params = {'zip5': zip5, 'zip4': zip4}
        result = self.api.api_call('District.getByZip', params)
        return self.result_to_obj(DistrictData, result['districtList']['district'])



"""
Measure methods that correspond to this API documentation page

http://api.votesmart.org/docs/Measure.html
"""

from .base import APIMethodBase

class Measure(APIMethodBase):
    def __init__(self, api_instance):
        raise NotImplementedError()
#     class Measure(object):
#         @staticmethod
#         def getMeasuresByYearState(year, stateId):
#             params = {'year':year, 'stateId':stateId}
#             result = votesmart._apicall('Measure.getMeasuresByYearState', params)
#             return _result_to_obj(Measure, result['measures']['measure'])
#
#         @staticmethod
#         def getMeasure(measureId):
#             params = {'measureId':measureId}
#             result = votesmart._apicall('Measure.getMeasure', params)
#             return MeasureDetail(result['measure'])
#



"""
Rating methods that correspond to this API documentation page

http://api.votesmart.org/docs/Rating.html
"""

from .base import APIMethodBase
from .containers import VotesmartApiObject

class Category(VotesmartApiObject):
    def __str__(self):
        return ': '.join((self.categoryId, self.name))

class Sig(VotesmartApiObject):
    def __str__(self):
        return ': '.join((self.sigId, self.name))

class SigDetail(VotesmartApiObject):
    def __str__(self):
        return self.name

class RatingObject(VotesmartApiObject):
    def __str__(self):
        return self.ratingText

class Rating(APIMethodBase):

    def getCategories(self, stateId=None):
        params = {'stateId':stateId}
        result = self.api.api_call('Rating.getCategories', params)
        return self.result_to_obj(Category, result['categories']['category'])

    def getSigList(self, categoryId, stateId=None):
        params = {'categoryId':categoryId, 'stateId':stateId}
        result = self.api.api_call('Rating.getSigList', params)
        return self.result_to_obj(Sig, result['sigs']['sig'])

    def getSig(self, sigId):
        params = {'sigId':sigId}
        result = self.api.api_call('Rating.getSig', params)
        return SigDetail(result['sig'])

    def getSigRatings(self, sigId):
        params = {'sigId':sigId}
        result = self.api.api_call('Rating.getSigRatings', params)
        #TODO Fix this
        return VotesmartApiObject(result['sigRatings'])

    def getCandidateRating(self, candidateId, sigId=None):
        params = {'candidateId':candidateId, 'sigId':sigId}
        result = self.api.api_call('Rating.getCandidateRating', params)
        return self.result_to_obj(RatingObject, result['candidateRating']['rating'])

    def getRating(self, ratingId):
        params = {'ratingId':ratingId}
        result = self.api.api_call('Rating.getRating', params)
        return self.result_to_obj(VotesmartApiObject, result['rating']['candidateRating'])



from votesmart.methods.containers import VotesmartApiObject
from votesmart.methods.base import *

def test_result_to_obj():
    data = {}
    data['address'] = '123 address'
    data['phone'] = ['623-232-2321']
    data['notes'] = [" a note"]

    obj = APIMethodBase(api_instance="fake instance")

    result = obj.result_to_obj(VotesmartApiObject, data)
    assert 1 == len(result)
    assert result[0].address == '123 address'



"""Command functions instances for API Calls
"""


class APIMethodBase:
    def __init__(self, api_instance):
        self.api = api_instance

    def result_to_obj(self, cls, result):
        "Convert a dict / list response into a list of parsed elements"
        if isinstance(result, dict):
            return [cls(result)]
        else:
            # the if o predicate is important, sometimes they return empty strings
            return [cls(o) for o in result if o]



"""
Leadership methods that correspond to this API documentation page

http://api.votesmart.org/docs/Leadership.html
"""

from .base import APIMethodBase

class Leadership(APIMethodBase):
    def __init__(self, api_instance):
        raise NotImplementedError()
#     class Leadership(object):
#         @staticmethod
#         def getPositions(stateId=None, officeId=None):
#             params = {'stateId':stateId, 'officeId':officeId}
#             result = votesmart._apicall('Leadership.getPositions', params)
#             return _result_to_obj(LeadershipPosition, result['leadership']['position'])
#
#         @staticmethod
#         def getOfficials(leadershipId, stateId=None):
#            params = {'leadershipId':leadershipId, 'stateId':stateId}
#            result = votesmart._apicall('Leadership.getOfficials', params)
#            return result['leaders']['leader']



import os
import re
import codecs

from setuptools import setup
from setuptools import find_packages

PROJECT = os.path.abspath(os.path.dirname(__file__))
REQUIRE_PATH = "requirements.txt"
EXCLUDES     = (
    "tests", "bin", "docs", "fixtures", "register", "notebooks", "examples",
)

long_description = open('README.rst').read()

def read(*parts):
    """
    Assume UTF-8 encoding and return the contents of the file located at the
    absolute path from the REPOSITORY joined with *parts.
    """
    with codecs.open(os.path.join(PROJECT, *parts), 'rb', 'utf-8') as f:
        return f.read()

def get_requires(path=REQUIRE_PATH):
    """
    Yields a generator of requirements as defined by the REQUIRE_PATH which
    should point to a requirements.txt output by `pip freeze`.
    """
    for line in read(path).splitlines():
        line = line.strip()
        if line and not line.startswith('#'):
            yield line

setup(name="py-votesmart",
      version="0.4.3",
      description="Libraries for interacting with the Project Vote Smart API",
      author="Nathan Danielsen <nathan.danielsen@gmail.com>",
      author_email = "nathan.danielsen@gmail.com",
      license="BSD",
      url="http://github.com/ndanielsen/py-votesmart/",
      long_description="py-votesmart is a fork of the original python-votesmart with python3 support",
      packages=find_packages(where=PROJECT, exclude=EXCLUDES),

      platforms=["any"],
      classifiers=["Development Status :: 3 - Alpha",
                   "Intended Audience :: Developers",
                   "License :: OSI Approved :: BSD License",
                   "Natural Language :: English",
                   "Operating System :: OS Independent",
                   "Programming Language :: Python",
                   "Topic :: Software Development :: Libraries :: Python Modules",
                   'Programming Language :: Python',
                   "Programming Language :: Python :: 2.7",
                   "Programming Language :: Python :: 3.4",
                   "Programming Language :: Python :: 3.5",
                   "Programming Language :: Python :: 3.6",
                   ],
      install_requires=list(get_requires()),
      )



"""
CandidateBio methods that correspond to this API documentation page

http://api.votesmart.org/docs/CandidateBio.html
"""

from .base import APIMethodBase
from .containers import VotesmartApiObject

class Bio(VotesmartApiObject):
    def __init__(self, d):
        self.__dict__.update(d['candidate'])

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, self.__dict__)

class AddlBio(VotesmartApiObject):
    def __str__(self):
        return ': '.join((self.name, self.data))


class CandidateBio(APIMethodBase):
    def getBio(self, candidateId):
        params = {'candidateId': candidateId}
        result = self.api.api_call('CandidateBio.getBio', params)
        return Bio(result['bio'])

    def getDetailedBio(self, candidateId):
        params = {'candidateId': candidateId}
        result = self.api.api_call('CandidateBio.getDetailedBio', params)
        return Bio(result['bio'])

    def getAddlBio(self, candidateId):
        params = {'candidateId': candidateId}
        result = self.api.api_call('CandidateBio.getAddlBio', params)
        return self.result_to_obj(AddlBio,
                              result['addlBio']['additional']['item'])



# """ Python library for interacting with Project Vote Smart API.
#
#     Project Vote Smart's API (http://www.votesmart.org/services_api.php)
#     provides rich biographical data, including data on votes, committee
#     assignments, and much more.
# """
#
# from .exceptions import VotesmartApiError
# from .methods.containers import VotesmartApiObject
# from .api import VoteSmartAPI
#
# try:
#     from urllib2 import urlopen, HTTPError
# except ImportError:
#     from urllib.request import urlopen, HTTPError
#
# try:
#     from urllib import urlencode
# except ImportError:
#     from urllib.parse import urlencode
#
# try:
#     import json
# except ImportError:
#     import simplejson as json
#
# from .utils import _result_to_obj
#
#
# class AddressData(object):
#     def __init__(self, d):
#         self.__dict__ = d
#         # self.__dict__.update(d['address'])
#         # self.__dict__.update(d['phone'])
#         # self.__dict__.update(d['notes'])
#
#     def __repr__(self):
#         return '%s(%r)' % (self.__class__.__name__, self.__dict__)
#
# class WebAddress(VotesmartApiObject):
#     def __str__(self):
#         return self.webAddress
#
# class Bio(object):
#     def __init__(self, d):
#         #self.__dict__.update(d['election'])
#         #self.__dict__.update(d['office'])
#         self.__dict__.update(d['candidate'])
#
#     def __repr__(self):
#         return '%s(%r)' % (self.__class__.__name__, self.__dict__)
#
# class AddlBio(VotesmartApiObject):
#     def __str__(self):
#         return ': '.join((self.name, self.data))
#
# class Candidate(VotesmartApiObject):
#     def __str__(self):
#         return ' '.join((self.firstName, self.lastName))
#
# class CommitteeType(VotesmartApiObject):
#     def __str__(self):
#         return self.name
#
# class Committee(VotesmartApiObject):
#     def __str__(self):
#         return self.name
#
# class CommitteeDetail(VotesmartApiObject):
#     def __str__(self):
#         return self.name
#
# class CommitteeMember(VotesmartApiObject):
#     def __str__(self):
#         return ' '.join((self.title, self.firstName, self.lastName))
#
# class District(VotesmartApiObject):
#     def __str__(self):
#         return self.name
#
# class Election(VotesmartApiObject):
#     def __init__(self, d):
#         stages = d.pop('stage', None)
#         self.__dict__ = d
#         if stages:
#             self.stages = _result_to_obj(ElectionStage, stages)
#
#     def __str__(self):
#         return self.name
#
# class ElectionStage(VotesmartApiObject):
#     def __str__(self):
#         return '%s (%s)' % (self.name, self.electionDate)
#
# class Official(VotesmartApiObject):
#     def __str__(self):
#         return ' '.join((self.title, self.firstName, self.lastName))
#
# class LeadershipPosition(VotesmartApiObject):
#     def __str__(self):
#         return self.name
#
# class Locality(VotesmartApiObject):
#     def __str__(self):
#         return self.name
#
# class Measure(VotesmartApiObject):
#     def __str__(self):
#         return self.title
#
# class MeasureDetail(VotesmartApiObject):
#     def __str__(self):
#         return self.title
#
# class OfficeType(VotesmartApiObject):
#     def __str__(self):
#         return ': '.join((self.officeTypeId, self.name))
#
# class OfficeBranch(VotesmartApiObject):
#     def __str__(self):
#         return ': '.join((self.officeBranchId, self.name))
#
# class OfficeLevel(VotesmartApiObject):
#     def __str__(self):
#         return ': '.join((self.officeLevelId, self.name))
#
# class Office(VotesmartApiObject):
#     def __str__(self):
#         return self.name
#
# class Category(VotesmartApiObject):
#     def __str__(self):
#         return ': '.join((self.categoryId, self.name))
#
# class Sig(VotesmartApiObject):
#     def __str__(self):
#         return ': '.join((self.sigId, self.name))
#
# class SigDetail(VotesmartApiObject):
#     def __str__(self):
#         return self.name
#
# class Rating(VotesmartApiObject):
#     def __str__(self):
#         return self.ratingText
#
# class State(VotesmartApiObject):
#     def __str__(self):
#         return ' '.join((self.stateId, self.name))
#
# class StateDetail(VotesmartApiObject):
#     def __str__(self):
#         return ' '.join((self.stateId, self.name))
#
# class BillSponsor(VotesmartApiObject):
#     def __str__(self):
#         return self.name
#
# class BillAction(VotesmartApiObject):
#     def __str__(self):
#         return ' - '.join((self.statusDate, self.stage))
#
# class BillAmendment(VotesmartApiObject):
#     def __str__(self):
#         return self.title
#
# class BillDetail(VotesmartApiObject):
#     def __init__(self, d):
#         sponsors = d.pop('sponsors')
#         actions = d.pop('actions')
#         amendments = d.pop('amendments')  # ammendments -- sic
#         if not sponsors: sponsors = { 'sponsor': [] }
#         self.sponsors = _result_to_obj(BillSponsor, sponsors['sponsor'])
#         self.actions = _result_to_obj(BillAction, actions['action'])
#         if amendments:
#             self.amendments = _result_to_obj(BillAmendment, amendments['amendment'])
#         self.__dict__.update(d)
#
# class BillActionDetail(VotesmartApiObject):
#     def __str__(self):
#         return self.officialTitle
#
# class Bill(VotesmartApiObject):
#     def __str__(self):
#         return ' '.join((self.billNumber, self.title))
#
# class Vote(VotesmartApiObject):
#     def __str__(self):
#         return ': '.join((self.candidateName, self.action))
#
# class Veto(VotesmartApiObject):
#     def __str__(self):
#         return ' '.join((self.billNumber, self.billTitle))
#
#
#
# class votesmart(object):
#
#     apikey = None
#
#     @staticmethod
#     def _apicall(func, params):
#         if votesmart.apikey is None:
#             raise VotesmartApiError('Missing Project Vote Smart apikey')
#
#         params = dict([(k,v) for (k,v) in params.items() if v])
#         url = 'http://api.votesmart.org/%s?o=JSON&key=%s&%s' % (func,
#             votesmart.apikey, urlencode(params))
#         try:
#             response = urlopen(url).read()
#             obj = json.loads(response.decode('utf-8'))
#             if 'error' in obj:
#                 raise VotesmartApiError(obj['error']['errorMessage'])
#             else:
#                 return obj
#         except HTTPError as e:
#             raise VotesmartApiError(e)
#         except ValueError as e:
#             raise VotesmartApiError('Invalid Response')
#
#     class Address(object):
#         @staticmethod
#         def getCampaign(candidateId):
#             params = {'candidateId': candidateId}
#             result = votesmart._apicall('Address.getCampaign', params)
#             return _result_to_obj(AddressData, result['address']['office'])
#
#         @staticmethod
#         def getCampaignWebAddress(candidateId):
#             params = {'candidateId': candidateId}
#             result = votesmart._apicall('Address.getCampaignWebAddress', params)
#             return _result_to_obj(WebAddress, result['webaddress']['address'])
#
#         @staticmethod
#         def getCampaignByElection(electionId):
#             params = {'electionId': electionId}
#             result = votesmart._apicall('Address.getCampaignByElection', params)
#             return _result_to_obj(AddressData, result['address']['office'])
#
#         @staticmethod
#         def getOffice(candidateId):
#             params = {'candidateId': candidateId}
#             result = votesmart._apicall('Address.getOffice', params)
#             return _result_to_obj(AddressData, result['address']['office'])
#
#         @staticmethod
#         def getOfficeWebAddress(candidateId):
#             params = {'candidateId': candidateId}
#             result = votesmart._apicall('Address.getOfficeWebAddress', params)
#             return _result_to_obj(WebAddress, result['webaddress']['address'])
#
#         #@staticmethod
#         #def getOfficeByOfficeState(officeId, stateId=None):
#         #    params = {'officeId': officeId, 'stateId': stateId}
#         #    result = votesmart._apicall('Address.getOfficeByOfficeState', params)
#         #    return _result_to_obj(Address, result['address']['office'])
#
#     class CandidateBio(object):
#         @staticmethod
#         def getBio(candidateId):
#             params = {'candidateId': candidateId}
#             result = votesmart._apicall('CandidateBio.getBio', params)
#             return Bio(result['bio'])
#
#         @staticmethod
#         def getAddlBio(candidateId):
#             params = {'candidateId': candidateId}
#             result = votesmart._apicall('CandidateBio.getAddlBio', params)
#             return _result_to_obj(AddlBio,
#                                   result['addlBio']['additional']['item'])
#
#     class Candidates(object):
#         @staticmethod
#         def getByOfficeState(officeId, stateId=None, electionYear=None):
#             params = {'officeId': officeId, 'stateId':stateId, 'electionYear': electionYear}
#             result = votesmart._apicall('Candidates.getByOfficeState', params)
#             return _result_to_obj(Candidate, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByOfficeTypeState(officeTypeId, stateId=None, electionYear=None):
#             params = {'officeTypeId': officeTypeId, 'stateId':stateId, 'electionYear': electionYear}
#             result = votesmart._apicall('Candidates.getByOfficeTypeState', params)
#             return _result_to_obj(Candidate, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByLastname(lastName, electionYear=None):
#             params = {'lastName': lastName, 'electionYear':electionYear}
#             result = votesmart._apicall('Candidates.getByLastname', params)
#             return _result_to_obj(Candidate, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByLevenstein(lastName, electionYear=None):
#             params = {'lastName': lastName, 'electionYear':electionYear}
#             result = votesmart._apicall('Candidates.getByLevenstein', params)
#             return _result_to_obj(Candidate, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByElection(electionId):
#             params = {'electionId': electionId}
#             result = votesmart._apicall('Candidates.getByElection', params)
#             return _result_to_obj(Candidate, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByDistrict(districtId, electionYear=None):
#             params = {'districtId': districtId, 'electionYear':electionYear}
#             result = votesmart._apicall('Candidates.getByDistrict', params)
#             return _result_to_obj(Candidate, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByZip(zip5, zip4=None):
#             params = {'zip4': zip4, 'zip5': zip5}
#             result = votesmart._apicall('Candidates.getByZip', params)
#             return _result_to_obj(Candidate, result['candidateList']['candidate'])
#
#     class Committee(object):
#         @staticmethod
#         def getTypes():
#             result = votesmart._apicall('Committee.getTypes', {})
#             return _result_to_obj(CommitteeType, result['committeeTypes']['type'])
#
#         @staticmethod
#         def getCommitteesByTypeState(typeId=None, stateId=None):
#             params = {'typeId':typeId, 'stateId':stateId}
#             result = votesmart._apicall('Committee.getCommitteesByTypeState', params)
#             return _result_to_obj(Committee, result['committees']['committee'])
#
#         @staticmethod
#         def getCommittee(committeeId):
#             params = {'committeeId' : committeeId}
#             result = votesmart._apicall('Committee.getCommittee', params)
#             return CommitteeDetail(result['committee'])
#
#         @staticmethod
#         def getCommitteeMembers(committeeId):
#             params = {'committeeId' : committeeId}
#             result = votesmart._apicall('Committee.getCommitteeMembers', params)
#             return _result_to_obj(CommitteeMember, result['committeeMembers']['member'])
#
#     class District(object):
#         @staticmethod
#         def getByOfficeState(officeId, stateId, districtName=None):
#             params = {'officeId':officeId, 'stateId': stateId, 'districtName': districtName}
#             result = votesmart._apicall('District.getByOfficeState', params)
#             return _result_to_obj(District, result['districtList']['district'])
#
#         @staticmethod
#         def getByZip(zip5, zip4=None):
#             params = {'zip5': zip5, 'zip4': zip4}
#             result = votesmart._apicall('District.getByZip', params)
#             return _result_to_obj(District, result['districtList']['district'])
#
#     class Election(object):
#         @staticmethod
#         def getElection(electionId):
#             params = {'electionId':electionId}
#             result = votesmart._apicall('Election.getElection', params)
#             return Election(result['elections']['election'])
#
#         @staticmethod
#         def getElectionByYearState(year, stateId=None):
#             params = {'year':year, 'stateId':stateId}
#             result = votesmart._apicall('Election.getElectionByYearState', params)
#             return _result_to_obj(Election, result['elections']['election'])
#
#         @staticmethod
#         def getElectionByZip(zip5, zip4=None, year=None):
#             params = {'zip5': zip5, 'zip4': zip4, 'year': year}
#             result = votesmart._apicall('Election.getElectionByZip', params)
#             return _result_to_obj(Election, result['elections']['election'])
#
#         @staticmethod
#         def getStageCandidates(electionId, stageId, party=None,
#                                districtId=None, stateId=None):
#             params = {'electionId':electionId, 'stageId':stageId,
#                       'party':party, 'districtId':districtId, 'stateId':stateId}
#             result = votesmart._apicall('Election.getStageCandidates', params)
#             return _result_to_obj(Candidate, result['stageCandidates']['candidate'])
#
#
#     class Leadership(object):
#         @staticmethod
#         def getPositions(stateId=None, officeId=None):
#             params = {'stateId':stateId, 'officeId':officeId}
#             result = votesmart._apicall('Leadership.getPositions', params)
#             return _result_to_obj(LeadershipPosition, result['leadership']['position'])
#
#         @staticmethod
#         def getOfficials(leadershipId, stateId=None):
#            params = {'leadershipId':leadershipId, 'stateId':stateId}
#            result = votesmart._apicall('Leadership.getOfficials', params)
#            return result['leaders']['leader']
#
#     class Local(object):
#         @staticmethod
#         def getCounties(stateId):
#             params = {'stateId': stateId}
#             result = votesmart._apicall('Local.getCounties', params)
#             return _result_to_obj(Locality, result['counties']['county'])
#
#         @staticmethod
#         def getCities(stateId):
#             params = {'stateId': stateId}
#             result = votesmart._apicall('Local.getCities', params)
#             return _result_to_obj(Locality, result['cities']['city'])
#
#         @staticmethod
#         def getOfficials(localId):
#             params = {'localId': localId}
#             result = votesmart._apicall('Local.getOfficials', params)
#             return _result_to_obj(Official, result['candidateList']['candidate'])
#
#     class Measure(object):
#         @staticmethod
#         def getMeasuresByYearState(year, stateId):
#             params = {'year':year, 'stateId':stateId}
#             result = votesmart._apicall('Measure.getMeasuresByYearState', params)
#             return _result_to_obj(Measure, result['measures']['measure'])
#
#         @staticmethod
#         def getMeasure(measureId):
#             params = {'measureId':measureId}
#             result = votesmart._apicall('Measure.getMeasure', params)
#             return MeasureDetail(result['measure'])
#
#     class Npat(object):
#         @staticmethod
#         def getNpat(candidateId):
#             params = {'candidateId':candidateId}
#             result = votesmart._apicall('Npat.getNpat', params)
#             return result['npat']
#
#     class Office(object):
#         @staticmethod
#         def getTypes():
#             result = votesmart._apicall('Office.getTypes', {})
#             return _result_to_obj(OfficeType, result['officeTypes']['type'])
#
#         @staticmethod
#         def getBranches():
#             result = votesmart._apicall('Office.getBranches', {})
#             return _result_to_obj(OfficeBranch, result['branches']['branch'])
#
#         @staticmethod
#         def getLevels():
#             result = votesmart._apicall('Office.getLevels', {})
#             return _result_to_obj(OfficeLevel, result['levels']['level'])
#
#         @staticmethod
#         def getOfficesByType(typeId):
#             params = {'officeTypeId':typeId}
#             result = votesmart._apicall('Office.getOfficesByType', params)
#             return _result_to_obj(Office, result['offices']['office'])
#
#         @staticmethod
#         def getOfficesByLevel(levelId):
#             params = {'levelId':levelId}
#             result = votesmart._apicall('Office.getOfficesByLevel', params)
#             return _result_to_obj(Office, result['offices']['office'])
#
#         @staticmethod
#         def getOfficesByTypeLevel(typeId, levelId):
#             params = {'officeTypeId':typeId, 'officeLevelId':levelId}
#             result = votesmart._apicall('Office.getOfficesByTypeLevel', params)
#             return _result_to_obj(Office, result['offices']['office'])
#
#         @staticmethod
#         def getOfficesByBranchLevel(branchId, levelId):
#             params = {'branchId':branchId, 'levelId':levelId}
#             result = votesmart._apicall('Office.getOfficesByBranchLevel', params)
#             return _result_to_obj(Office, result['offices']['office'])
#
#     class Officials(object):
#         @staticmethod
#         def getStatewide(stateId=None):
#             params = {'stateId': stateId}
#             result = votesmart._apicall('Officials.getStatewide', params)
#             return _result_to_obj(Official, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByOfficeState(officeId, stateId=None):
#             params = {'officeId':officeId, 'stateId': stateId}
#             result = votesmart._apicall('Officials.getByOfficeState', params)
#             return _result_to_obj(Official, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByLastname(lastName):
#             params = {'lastName':lastName}
#             result = votesmart._apicall('Officials.getByLastname', params)
#             return _result_to_obj(Official, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByLevenstein(lastName):
#             params = {'lastName':lastName}
#             result = votesmart._apicall('Officials.getByLevenstein', params)
#             return _result_to_obj(Official, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByElection(electionId):
#             params = {'electionId':electionId}
#             result = votesmart._apicall('Officials.getByElection', params)
#             return _result_to_obj(Official, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByDistrict(districtId):
#             params = {'districtId':districtId}
#             result = votesmart._apicall('Officials.getByDistrict', params)
#             return _result_to_obj(Official, result['candidateList']['candidate'])
#
#         @staticmethod
#         def getByZip(zip5, zip4=None):
#             params = {'zip4': zip4, 'zip5': zip5}
#             result = votesmart._apicall('Officials.getByZip', params)
#             return _result_to_obj(Official, result['candidateList']['candidate'])
#
#     class Rating(object):
#         @staticmethod
#         def getCategories(stateId=None):
#             params = {'stateId':stateId}
#             result = votesmart._apicall('Rating.getCategories', params)
#             return _result_to_obj(Category, result['categories']['category'])
#
#         @staticmethod
#         def getSigList(categoryId, stateId=None):
#             params = {'categoryId':categoryId, 'stateId':stateId}
#             result = votesmart._apicall('Rating.getSigList', params)
#             return _result_to_obj(Sig, result['sigs']['sig'])
#
#         @staticmethod
#         def getSig(sigId):
#             params = {'sigId':sigId}
#             result = votesmart._apicall('Rating.getSig', params)
#             return SigDetail(result['sig'])
#
#         @staticmethod
#         def getCandidateRating(candidateId, sigId=None):
#             params = {'candidateId':candidateId, 'sigId':sigId}
#             result = votesmart._apicall('Rating.getCandidateRating', params)
#             return _result_to_obj(Rating, result['candidateRating']['rating'])
#
#     class State(object):
#         @staticmethod
#         def getStateIDs():
#             result = votesmart._apicall('State.getStateIDs', {})
#             return _result_to_obj(State, result['stateList']['list']['state'])
#
#         @staticmethod
#         def getState(stateId):
#             params = {'stateId':stateId}
#             result = votesmart._apicall('State.getState', params)
#             return StateDetail(result['state']['details'])
#
#     class Votes(object):
#         @staticmethod
#         def getCategories(year, stateId=None):
#             params = {'year':year, 'stateId':stateId}
#             result = votesmart._apicall('Votes.getCategories', params)
#             return _result_to_obj(Category, result['categories']['category'])
#
#         @staticmethod
#         def getBill(billId):
#             params = {'billId':billId}
#             result = votesmart._apicall('Votes.getBill', params)
#             return BillDetail(result['bill'])
#
#         @staticmethod
#         def getBillAction(actionId):
#             params = {'actionId':actionId}
#             result = votesmart._apicall('Votes.getBillAction', params)
#             return BillActionDetail(result['action'])
#
#         @staticmethod
#         def getBillActionVotes(actionId):
#             params = {'actionId':actionId}
#             result = votesmart._apicall('Votes.getBillActionVotes', params)
#             return _result_to_obj(Vote, result['votes']['vote'])
#
#         @staticmethod
#         def getBillActionVoteByOfficial(actionId, candidateId):
#             params = {'actionId':actionId, 'candidateId':candidateId}
#             result = votesmart._apicall('Votes.getBillActionVoteByOfficial', params)
#             return Vote(result['votes']['vote'])
#
#         @staticmethod
#         def getByBillNumber(billNumber):
#             params = {'billNumber': billNumber}
#             result = votesmart._apicall('Votes.getByBillNumber', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getBillsByCategoryYearState(categoryId, year, stateId=None):
#             params = {'categoryId':categoryId, 'year':year, 'stateId':stateId}
#             result = votesmart._apicall('Votes.getBillsByCategoryYearState', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getBillsByYearState(year, stateId=None):
#             params = {'year':year, 'stateId':stateId}
#             result = votesmart._apicall('Votes.getBillsByYearState', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getBillsByOfficialYearOffice(candidateId, year, officeId=None):
#             params = {'candidateId':candidateId, 'year':year, 'officeId':officeId}
#             result = votesmart._apicall('Votes.getBillsByOfficialYearOffice', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getBillsByOfficial(candidateId, year, officeId=None, categoryId=None):
#             params = {'candidateId':candidateId, 'year':year, 'officeId':officeId, 'categoryId':categoryId}
#             result = votesmart._apicall('Votes.getBillsByOfficial', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getBillsByOfficialCategoryOffice(candidateId, categoryId, officeId=None):
#             params = {'candidateId':candidateId, 'categoryId':categoryId, 'officeId':officeId}
#             result = votesmart._apicall('Votes.getBillsByOfficialCategoryOffice', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getBillsBySponsorYear(candidateId, year):
#             params = {'candidateId':candidateId, 'year':year}
#             result = votesmart._apicall('Votes.getBillsBySponsorYear', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getBillsBySponsorCategory(candidateId, categoryId):
#             params = {'candidateId':candidateId, 'categoryId':categoryId}
#             result = votesmart._apicall('Votes.getBillsBySponsorCategory', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getBillsByStateRecent(stateId=None, amount=None):
#             params = {'stateId':stateId, 'amount':amount}
#             result = votesmart._apicall('Votes.getBillsByStateRecent', params)
#             return _result_to_obj(Bill, result['bills']['bill'])
#
#         @staticmethod
#         def getVetoes(candidateId):
#             params = {'candidateId': candidateId}
#             result = votesmart._apicall('Votes.getVetoes', params)
#             return _result_to_obj(Veto, result['vetoes']['veto'])
